/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Application.h"
#include "ConfigurationDialog.h"
#include "ConnectionObject.h"
#include "File.h"
#include "IconNames.h"
#include "IconEngine.h"
#include "LogDialog.h"
#include "MainMenu.h"
#include "SystemTrayIcon.h"
#include "Util.h"
#include "XmlOptions.h"
#include "CustomProcess.h"

#include <QApplication>
#include <QDesktopWidget>

//____________________________________________
Application::Application( CommandLineArguments arguments ):
    BaseApplication( nullptr, arguments ),
    Counter( "Application" )
{}

//____________________________________________
Application::~Application() = default;

//____________________________________________
bool Application::realizeWidget()
{
    Debug::Throw( "Application::realizeWidget.\n" );

    // check if the method has already been called.
    if( !BaseApplication::realizeWidget() ) return false;

    // log
    showLogAction_ = new QAction( IconEngine::get( IconNames::Find ), tr( "Show Log" ), this );

    // top widget
    systemTrayIcon_.reset( new SystemTrayIcon );

    // log
    logDialog_.reset( new LogDialog );

    connect( &systemTrayIcon_->mainMenu(), SIGNAL(messageAvailable(QString)), logDialog_.get(), SLOT(processMessage(QString)) );
    connect( showLogAction_, SIGNAL(triggered()), logDialog_.get(), SLOT(show()) );

    // show main window
    emit configurationChanged();
    systemTrayIcon_->show();

    // timer is needed so that geometry is set properly at first connection
    // use 2 seconds delay. This is arbitrary.
    timer_.start( 2000, this );

    return true;

}


//____________________________________________
CommandLineParser Application::commandLineParser( CommandLineArguments arguments, bool ignoreWarnings ) const
{

    Debug::Throw( "Application::commandLineParser.\n" );
    CommandLineParser out( BaseApplication::commandLineParser() );
    out.setGroup( CommandLineParser::applicationGroupName );
    out.registerFlag( CommandLineParser::Tag( "--list", "-l" ), tr( "list all configured connections" ) );
    out.registerOption( CommandLineParser::Tag( "--connect" ), tr( "string" ), tr( "try connect specified connection" ) );
    out.registerOption( CommandLineParser::Tag( "--disconnect" ), tr( "string" ), tr( "try disconnect specified connection" ) );
    out.registerOption( CommandLineParser::Tag( "--reconnect" ), tr( "string" ), tr( "try re-connect specified connection" ) );

    if( !arguments.isEmpty() ) out.parse( arguments, ignoreWarnings );
    return out;

}

//____________________________________________
void Application::usage() const
{
    _usage( "qssh", tr( "[options] [host]" ) );
    commandLineParser().usage();
    return;
}

//___________________________________________________________
void Application::_configuration()
{
    Debug::Throw( "Application::_configuration.\n" );
    emit saveConfiguration();
    ConfigurationDialog dialog( nullptr, systemTrayIcon_->supportsSystemNotifications() );
    connect( &dialog, SIGNAL(configurationChanged()), SIGNAL(configurationChanged()) );
    dialog.centerOnWidget( qApp->activeWindow() );
    dialog.exec();

}

//___________________________________________________________
void Application::timerEvent( QTimerEvent* event )
{
    if( event->timerId() == timer_.timerId() )
    {
        timer_.stop();
        const CommandLineParser parser( commandLineParser( _arguments() ) );
        if( parser.hasOption( "--connect" ) ) systemTrayIcon_->mainMenu().connect( parser.option( "--connect" ) );
        else if( parser.hasOption( "--reconnect" ) ) systemTrayIcon_->mainMenu().reconnect( parser.option( "--reconnect" ) );
        else if( !parser.hasOption( "--disconnect" ) ) systemTrayIcon_->mainMenu().autoConnect();
    }
}

//________________________________________________
bool Application::_processCommand( Server::ServerCommand command )
{

    Debug::Throw( "Application::_processCommand.\n" );
    if( BaseApplication::_processCommand( command ) ) return true;
    if( command.command() == Server::ServerCommand::Raise )
    {

        const CommandLineParser parser( commandLineParser( command.arguments() ) );
        if( parser.hasOption( "--connect" ) ) systemTrayIcon_->mainMenu().connect( parser.option( "--connect" ) );
        else if( parser.hasOption( "--reconnect" ) ) systemTrayIcon_->mainMenu().reconnect( parser.option( "--reconnect" ) );
        else if( parser.hasOption( "--disconnect" ) ) systemTrayIcon_->mainMenu().disconnect( parser.option( "--disconnect" ) );
        else {

            // in all other cases find all objects and list which are connected and which are not
            // find connection object
            QList<ConnectionObject*> objects( systemTrayIcon_->mainMenu().findChildren<ConnectionObject*>() );
            if( !objects.empty() )
            {

                QString notification;
                for( const auto& object:objects )
                {
                    notification += QString( "%1: %2\n" )
                        .arg( object->attributes().name() )
                        .arg( object->connectAction().isChecked() ? tr( "connected" ):tr( "disconnected" ) );
                }

                systemTrayIcon_->notify( notification );
            }

        }
        return true;

    } else return false;

}
