/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "ConnectionAttributesModel.h"

#include "CppUtil.h"
#include "Debug.h"
#include "IconEngine.h"
#include "IconNames.h"

#include <QMimeData>

//______________________________________________________________________
ConnectionAttributesModel::ConnectionAttributesModel( QObject* parent ):
    ListModel( parent ),
    Counter( "ConnectionDialog::ConnectionAttributesModel" ),
    icon_( IconEngine::get( IconNames::Connection ) )
{}

//__________________________________________________________________
Qt::ItemFlags ConnectionAttributesModel::flags(const QModelIndex &index) const
{
    if( !index.isValid() ) return Qt::ItemIsDropEnabled;
    return Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsDragEnabled;
}

//______________________________________________________________________
QVariant ConnectionAttributesModel::data( const QModelIndex& index, int role ) const
{
    Debug::Throw( "ConnectionAttributesModel::data.\n" );

    // check index
    if( !contains( index ) ) return QVariant();

    // retrieve associated file info
    const Ssh::ConnectionAttributes& attributes( get()[index.row()] );

    // return text associated to file and column
    if( role == Qt::DisplayRole ) {

        switch( index.column() )
        {
            case Name: return attributes.name();
            case Host: return attributes.host();
            case User: return attributes.userName();
            default: return QVariant();
        }

    } else if( role == Qt::DecorationRole && index.column() == Name ) {

        return icon_;

    }

    return QVariant();

}

//______________________________________________________________________
QVariant ConnectionAttributesModel::headerData(int section, Qt::Orientation orientation, int role ) const
{

    if(
        orientation == Qt::Horizontal &&
        role == Qt::DisplayRole &&
        section >= 0 &&
        section < nColumns )
    { return columnTitles_[section]; }

    // return empty
    return QVariant();

}

//______________________________________________________________________
QStringList ConnectionAttributesModel::mimeTypes() const
{
    static const auto types = Base::makeT<QStringList>( { Ssh::ConnectionAttributes::MimeType, "text/plain" } );
    return types;
}

//______________________________________________________________________
QMimeData* ConnectionAttributesModel::mimeData(const QModelIndexList &indexes) const
{

    // create mime data
    QMimeData *mime = new QMimeData;

    // add keywords mimetype
    for( const auto& index:indexes )
    { if( index.isValid() && index.column() == Name ) mime->setData( Ssh::ConnectionAttributes::MimeType, qPrintable( QString::number(index.row()) ) ); }

    // add plain text mimetype
    QString buffer;
    for( const auto& index:indexes )
    {
        if( !( index.isValid() && index.column() == Name ) ) continue;
        const Ssh::ConnectionAttributes& data( get( index ) );
        buffer += data.name();
    }

    // set plain text data
    mime->setData( "text/plain", qPrintable( buffer ) );

    return mime;

}

//__________________________________________________________________
bool ConnectionAttributesModel::dropMimeData(const QMimeData* data , Qt::DropAction action, int row, int column, const QModelIndex& parent)
{

    Debug::Throw( "ConnectionAttributesModel::dropMimeData\n" );

    // check action
    if( action == Qt::IgnoreAction) return true;
    if( row < 0 ) return false;
    if( data->hasFormat( Ssh::ConnectionAttributes::MimeType ) )
    {

        // retrieve row of dragged item and send signal
        const int oldRow( data->data( Ssh::ConnectionAttributes::MimeType ).toInt() );
        emit itemOrderChanged( oldRow, row );
        return true;

    }

    return false;
}
