/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "ConnectionListWidget.h"

#include "IconEngine.h"
#include "IconNames.h"
#include "SimpleListView.h"

#include <QLabel>
#include <QLayout>

//___________________________________________________________
ConnectionListWidget::ConnectionListWidget( QWidget* parent ):
    QWidget(parent)
{

    Debug::Throw( "ConnectionListWidget::ConnectionListWidget.\n" );

    // horizontal layout
    QHBoxLayout* hLayout = new QHBoxLayout;
    hLayout->setMargin(0);
    hLayout->setSpacing(5);
    setLayout( hLayout );

    hLayout->addWidget( list_ = new SimpleListView( this ), 0 );
    list_->setProperty( "_kde_side_panel_view", true );
    list_->setModel( &model_ );

    list_->setDragEnabled(true);
    list_->setAcceptDrops(true);

    QVBoxLayout* vLayout = new QVBoxLayout;
    vLayout->setMargin(0);
    vLayout->setSpacing(5);
    hLayout->addLayout( vLayout, 1 );

    // buttons
    vLayout->addWidget( addButton_ = new QPushButton( IconEngine::get( IconNames::Add ), tr( "Add" ), this ) );
    vLayout->addWidget( removeButton_ = new QPushButton( IconEngine::get( IconNames::Remove ), tr( "Remove" ), this ) );
    vLayout->addWidget( editButton_ = new QPushButton( IconEngine::get( IconNames::Edit ), tr( "Edit" ), this ) );
    vLayout->addWidget( moveUpButton_ = new QPushButton( IconEngine::get( IconNames::Up ), tr( "Move Up" ), this ) );
    vLayout->addWidget( moveDownButton_ = new QPushButton( IconEngine::get( IconNames::Down ), tr( "Move Down" ), this ) );
    vLayout->addStretch( 1 );

    addButton_->setToolTip( tr( "Add a new connection" ) );
    removeButton_->setToolTip( tr( "Remove selected connection" ) );
    editButton_->setToolTip( tr( "Edit selected connection properties" ) );
    moveUpButton_->setToolTip( tr( "Move selected connection up in the list" ) );
    moveDownButton_->setToolTip( tr( "Move selected connection down in the list" ) );

    // connections
    connect( &model_, SIGNAL(itemOrderChanged(int,int)), SLOT(_reorder(int,int)) );

    connect( removeButton_, SIGNAL(pressed()), SLOT(_remove()) );
    connect( moveUpButton_, SIGNAL(pressed()), SLOT(_up()) );
    connect( moveDownButton_, SIGNAL(pressed()), SLOT(_down()) );
    connect( list_, SIGNAL(activated(QModelIndex)), editButton_, SLOT(click()) );

    connect( list_->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)), SLOT(_updateButtons()) );
    connect( list_->selectionModel(), SIGNAL(currentChanged(QModelIndex,QModelIndex)), SLOT(_updateButtons()) );

    _updateButtons();

}

//________________________________________
void ConnectionListWidget::setAttributes( const Ssh::ConnectionAttributes::List& attributesList )
{
    Debug::Throw( "ConnectionListWidget::setAttributes.\n" );
    model_.set( attributesList );
}

//________________________________________
Ssh::ConnectionAttributes::List ConnectionListWidget::attributes() const
{
    Debug::Throw( "ConnectionListWidget::attributes.\n" );
    return model_.get();
}

//_________________________________________________________________________
bool ConnectionListWidget::isUnique( const Ssh::ConnectionAttributes& attributes ) const
{

    auto attributesList( model_.get() );
    return std::none_of( attributesList.begin(), attributesList.end(), Ssh::ConnectionAttributes::SameNameFTor( attributes.name() ) );

}

//________________________________________
void ConnectionListWidget::_updateButtons()
{
    Debug::Throw( "ConnectionListWidget::_updateButtons.\n" );

    bool hasSelection( !list_->selectionModel()->selectedIndexes().empty() );
    editButton_->setEnabled( hasSelection );
    removeButton_->setEnabled( hasSelection );

    moveUpButton_->setEnabled( hasSelection && !list_->selectionModel()->isSelected( model_.index( 0, 0 ) ) );
    moveDownButton_->setEnabled( hasSelection && !list_->selectionModel()->isSelected( model_.index( model_.rowCount()-1, 0 ) ) );

}

//________________________________________
void ConnectionListWidget::_remove()
{
    Debug::Throw( "ConnectionListWidget::_remove.\n" );
    model_.remove( model_.get( list_->selectionModel()->selectedIndexes() ) );
}

//_________________________________________________________________________
void ConnectionListWidget::_up()
{

    Debug::Throw( "ConnectionListWidget::_up.\n" );
    Ssh::ConnectionAttributes::List selection( model_.get( list_->selectionModel()->selectedIndexes() ) );
    if( selection.empty() ) return;

    // retrieve selected indexes in list and store in model
    QModelIndexList selectedIndexes( list_->selectionModel()->selectedIndexes() );
    Ssh::ConnectionAttributes::List selectedAttributes( model_.get( selectedIndexes ) );

    Ssh::ConnectionAttributes::List currentAttributes( attributes() );
    Ssh::ConnectionAttributes::List newAttributes;

    for( const auto& attribute:currentAttributes )
    {

        // check if new list is not empty, current index is selected and last index is not.
        // if yes, move.
        if(
            !( newAttributes.empty() ||
            selectedIndexes.indexOf( model_.index( attribute ) ) == -1 ||
            selectedIndexes.indexOf( model_.index( newAttributes.back() ) ) != -1
            ) )
        {
            Ssh::ConnectionAttributes last( newAttributes.back() );
            newAttributes.removeLast();
            newAttributes << attribute;
            newAttributes << last;
        } else newAttributes << attribute;

    }

    model_.set( newAttributes );

    // restore selection
    list_->selectionModel()->select( model_.index( selectedAttributes.front() ),  QItemSelectionModel::Clear|QItemSelectionModel::Select|QItemSelectionModel::Rows );
    for( const auto& attribute:selectedAttributes )
    { list_->selectionModel()->select( model_.index( attribute ), QItemSelectionModel::Select|QItemSelectionModel::Rows ); }

}

//_________________________________________________________________________
void ConnectionListWidget::_down()
{

    Debug::Throw( "ConnectionListWidget::_down.\n" );
    Ssh::ConnectionAttributes::List selection( model_.get( list_->selectionModel()->selectedIndexes() ) );
    if( selection.empty() ) return;

    // retrieve selected indexes in list and store in model
    QModelIndexList selectedIndexes( list_->selectionModel()->selectedIndexes() );
    auto selectedAttributes( model_.get( selectedIndexes ) );
    auto currentAttributes( ConnectionListWidget::attributes() );
    Ssh::ConnectionAttributes::List newAttributes;
    Ssh::ConnectionAttributes::ListIterator iter( currentAttributes );
    iter.toBack();
    while( iter.hasPrevious() )
    {
        const auto current( iter.previous() );

        // check if new list is not empty, current index is selected and last index is not.
        // if yes, move.
        if(
            !( newAttributes.empty() ||
            selectedIndexes.indexOf( model_.index( current ) ) == -1 ||
            selectedIndexes.indexOf( model_.index( newAttributes.front() ) ) != -1
            ) )
        {

            auto first( newAttributes.front() );
            newAttributes.removeFirst();
            newAttributes.prepend( current );
            newAttributes.prepend( first );

        } else newAttributes.prepend( current );
    }

    model_.set( newAttributes );

    // restore selection
    list_->selectionModel()->select( model_.index( selectedAttributes.front() ),  QItemSelectionModel::Clear|QItemSelectionModel::Select|QItemSelectionModel::Rows );
    for( const auto& attribute:selectedAttributes )
    { list_->selectionModel()->select( model_.index( attribute ), QItemSelectionModel::Select|QItemSelectionModel::Rows ); }

    return;
}

//_________________________________________________________
void ConnectionListWidget::_reorder( int oldRow, int newRow )
{

    Debug::Throw() << "ConnectionListWidget::_reorder - old: " << oldRow << " new: " << newRow << endl;
    if( newRow > oldRow ) newRow--;

    ConnectionAttributesModel::List dataList( model_.get() );
    dataList.move( oldRow, newRow );
    model_.set( dataList );

    // restore selection
    list_->selectionModel()->select( model_.index( newRow, 0 ),  QItemSelectionModel::Clear|QItemSelectionModel::Select|QItemSelectionModel::Rows );
    return;

}
