/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "ConnectionObject.h"

#include "CppUtil.h"
#include "Debug.h"
#include "InformationDialog.h"
#include "Operators.h"
#include "SshConnection.h"
#include "TimeStamp.h"
#include "XmlOptions.h"

#include <QLayout>
#include <QToolButton>

//_______________________________________________________________
ConnectionObject::ConnectionObject( QObject* parent ):
    QObject( parent ),
    Counter( "ConnectionObject" ),
    connection_( new Ssh::Connection( this ) )
{

    Debug::Throw( "ConnectionObject::ConnectionObject.\n" );

    // connection state
    connect( connection_, SIGNAL(attributesChanged()), SLOT(_updateAttributes()) );
    connect( connection_, SIGNAL(connected()), this, SIGNAL(connected()) );

    connect( connection_, SIGNAL(loginFailed()), SLOT(_notifyLoginFailed()));

    connect( connection_, SIGNAL(message(QString)), SLOT(_processMessage(QString)));
    connect( connection_, SIGNAL(error(QString)), SLOT(_processError(QString)));
    connect( connection_, SIGNAL(error(QString)), this, SIGNAL(disconnected()));

    // actions
    _installActions();

}

//_______________________________________________________________
ConnectionObject::~ConnectionObject()
{
    Debug::Throw( "ConnectionObject::~ConnectionObject.\n" );
    if( connection_ ) connection_->disconnect();
}

//____________________________________________________
void ConnectionObject::setAttributes( const Ssh::ConnectionAttributes& attributes )
{
    attributes_ = attributes;
    connection_->setAttributes( attributes_ );
    connectAction_->setText( attributes_.name() );
    connectAction_->setEnabled( true );
}

//_______________________________________________________________
void ConnectionObject::timerEvent( QTimerEvent* event )
{
    if( event->timerId() == timer_.timerId() )
    {

        timer_.stop();

        QString buffer;
        const bool state( connectAction_->isChecked() );
        if( savedState_ )
        {

            if( state ) buffer = tr( "%1: reconnected." ).arg( attributes_.name() );
            else buffer = tr( "%1: connection closed." ).arg( attributes_.name() );

        } else {

            if( state ) buffer = tr( "%1: connected." ).arg( attributes_.name() );
            else buffer = tr( "%1: connection failed." ).arg( attributes_.name() );

        }

        emit messageAvailable( buffer );

    } else return QObject::timerEvent( event );

}

//_______________________________________________________________
void ConnectionObject::_toggleConnection( bool state )
{
    Debug::Throw( "ConnectionObject::_toggleConnection.\n" );

    // store previous state and start timer
    if( !timer_.isActive() )
    {
        savedState_ = !state;
        timer_.start( 200, this );
    }

    if( state )
    {

        if( connection_->isDisconnected() )
        {
            connection_->createTunnels();
            connection_->connect();
            connection_->authenticate();
        }

    } else if( !connection_->isDisconnected() ) {

        connection_->disconnect();

    }

}

//______________________________________
void ConnectionObject::_updateAttributes()
{
    Debug::Throw() << "ConnectionObject::_updateAttributes." << endl;
    const Ssh::ConnectionAttributes& attributes( connection_->attributes() );
    if(
        attributes != attributes_ ||
        attributes_.rememberPassword() != attributes.rememberPassword() ||
        (attributes.rememberPassword() && attributes.password() != attributes_.password() ) )
    {
        Debug::Throw() << "ConnectionObject::_updateAttributes - attributes have changed" << endl;
        attributes_ = connection_->attributes();
        emit attributesChanged();
    }
}

//______________________________________
void ConnectionObject::_notifyLoginFailed()
{
    Debug::Throw(0, "ConnectionObject::_notifyLoginFailed.\n" );

    // notify login failed
    InformationDialog dialog( nullptr, tr( "Login failed" ) );
    dialog.setWindowTitle( tr( "SSH login" ) );
    dialog.exec();
    emit disconnected();
}


//______________________________________
void ConnectionObject::_processMessage(  const QString& message )
{
    QString fullMessage;
    QTextStream(&fullMessage)
        << TimeStamp::now().toString() << " - "
        << attributes_.userName() << "@" << attributes_.host() << ": "
        << message;

    emit messageAvailable( fullMessage, Log );
}

//______________________________________
void ConnectionObject::_processError(  const QString& errorMessage )
{

    QString fullMessage;
    QTextStream(&fullMessage)
        << TimeStamp::now().toString() << " - "
        << attributes_.userName() << "@" << attributes_.host() << ": "
        << errorMessage;

    emit messageAvailable( fullMessage, Error );

    // mark as disconnected
    connectAction_->setChecked(false);

}

//_______________________________________________________________
void ConnectionObject::_installActions()
{

    Debug::Throw( "ConnectionObject::_installActions.\n" );

    // actions
    connectAction_ = new QAction( tr( "Connect" ), this );
    connectAction_->setCheckable( true );
    connectAction_->setChecked( false );
    connectAction_->setToolTip( tr( "Toggle ssh connection" ) );
    connect( connectAction_, SIGNAL(toggled(bool)), SLOT(_toggleConnection(bool)) );

}
