/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "ConnectionParser.h"
#include "XmlDef.h"
#include "XmlDocument.h"

#include <QDomDocument>
#include <QDomElement>
#include <QFile>

//* Xml wrapper around Ssh::TunnelAttributes
class XmlTunnelAttributes: public Ssh::TunnelAttributes
{

    public:

    //* constructor
    explicit XmlTunnelAttributes()
    {}

    //* copy constructor
    explicit XmlTunnelAttributes( const Ssh::TunnelAttributes& other ):
        Ssh::TunnelAttributes( other )
    {}

    //* constructor from dom
    explicit XmlTunnelAttributes( const QDomElement& );

    //* dom element
    QDomElement domElement( QDomDocument& ) const;

};


//* Xml wrapper around Ssh::ConnectionAttributes
class XmlConnectionAttributes: public Ssh::ConnectionAttributes
{

    public:

    //* constructor
    explicit XmlConnectionAttributes()
    {}

    //* copy constructor
    explicit XmlConnectionAttributes( const Ssh::ConnectionAttributes& other ):
        Ssh::ConnectionAttributes( other )
    {}

    //* constructor from dom
    explicit XmlConnectionAttributes( const QDomElement& );

    //* dom element
    QDomElement domElement( QDomDocument& ) const;

};


//_________________________________________________________
XmlTunnelAttributes::XmlTunnelAttributes( const QDomElement& element )
{
    Debug::Throw( "XmlTunnelAttributes::XmlTunnelAttributes.\n" );

    // parse attributes
    QDomNamedNodeMap attributes( element.attributes() );
    for( int i=0; i<attributes.count(); i++ )
    {

        QDomAttr attribute( attributes.item( i ).toAttr() );
        if( attribute.isNull() ) continue;
        QString name( attribute.name() );
        QString value( attribute.value() );

        if( name == Xml::Host ) setHost( value );
        else if( name == Xml::LocalPort ) setLocalPort( value.toInt() );
        else if( name == Xml::RemotePort ) setRemotePort( value.toInt() );

    }
}

//_________________________________________________________
QDomElement XmlTunnelAttributes::domElement( QDomDocument& document ) const
{
    Debug::Throw( "TunnelAttributes::domElement.\n" );
    QDomElement out( document.createElement( Xml::Tunnel ) );

    // attributes
    out.setAttribute( Xml::Host, host() );
    out.setAttribute( Xml::LocalPort, QString::number( localPort() ) );
    out.setAttribute( Xml::RemotePort, QString::number( remotePort() ) );
    return out;

}

//_________________________________________________________
XmlConnectionAttributes::XmlConnectionAttributes( const QDomElement& element )
{
    Debug::Throw( "XmlConnectionAttributes::XmlConnectionAttributes.\n" );

    // parse attributes
    QDomNamedNodeMap attributes( element.attributes() );
    for( int i=0; i<attributes.count(); i++ )
    {

        QDomAttr attribute( attributes.item( i ).toAttr() );
        if( attribute.isNull() ) continue;
        QString name( attribute.name() );
        QString value( attribute.value() );

        if( name == Xml::Name ) setName( value );
        else if( name == Xml::Host ) setHost( value );
        else if( name == Xml::Port ) setPort( value.toInt() );
        else if( name == Xml::User ) setUserName( value );
        else if( name == Xml::AutoConnect ) setAutoConnect( value.toInt() );
        else if( name == Xml::RememberPassword ) setRememberPassword( value.toInt() );
        else if( name == Xml::Password ) setPassword( value );

    }

    // loop over children
    // parse children
    for(QDomNode child_node = element.firstChild(); !child_node.isNull(); child_node = child_node.nextSibling() )
    {
        QDomElement child_element = child_node.toElement();
        if( child_element.isNull() ) continue;
        if( child_element.tagName() == Xml::Tunnel ) addTunnel( XmlTunnelAttributes( child_element ) );
    }

}

//_________________________________________________________
QDomElement XmlConnectionAttributes::domElement( QDomDocument& document ) const
{
    Debug::Throw( "XmlConnectionAttributes::domElement.\n" );
    QDomElement out( document.createElement( Xml::Connection ) );

    // attributes
    out.setAttribute( Xml::Name, name() );
    out.setAttribute( Xml::Host, host() );
    out.setAttribute( Xml::Port, port() );
    out.setAttribute( Xml::User, userName() );
    out.setAttribute( Xml::AutoConnect, QString::number( autoConnect() ) );
    out.setAttribute( Xml::RememberPassword, QString::number( rememberPassword() ) );

    if( rememberPassword() && !password().isEmpty() )
    { out.setAttribute( Xml::Password, password() ); }

    // write tunnels
    for( const auto& tunnel:tunnels() )
    { out.appendChild( XmlTunnelAttributes( tunnel ).domElement( document ) ); }

    return out;

}

//_________________________________________________________
Ssh::ConnectionAttributes::List ConnectionParser::read()
{
    Debug::Throw( "ConnectionParser::read.\n" );
    Ssh::ConnectionAttributes::List out;

    // clear read error
    error_.clear();

    // parse the file
    XmlDocument document;
    {
        QFile qfile( file_ );
        if( !document.setContent( &qfile ) )
        {
            error_.clear();
            QTextStream( &error_ )
                << QObject::tr( "An error occured while parsing document classes." ) << endl
                << document.error()
                << endl;
            return out;
        }
    }

    // look for relevant element
    QDomNodeList topNodes = document.elementsByTagName( Xml::ConnectionList );
    if( topNodes.isEmpty() ) return out;

    for(QDomNode node = topNodes.at(0).firstChild(); !node.isNull(); node = node.nextSibling() )
    {
        QDomElement element = node.toElement();
        if( element.isNull() ) continue;

        if( element.tagName() == Xml::Connection )
        { out.append( XmlConnectionAttributes( element ) ); }

    }

    return out;

}


//_________________________________________________________
void ConnectionParser::write( const Ssh::ConnectionAttributes::List& attributesList )
{
    Debug::Throw( "ConnectionParser::write.\n" );

    // create document and read
    XmlDocument document;
    {
        QFile qtfile( file_ );
        document.setContent( &qtfile );
    }

    // create main element
    QDomElement top = document.createElement( Xml::ConnectionList );
    for( const auto& attributes:attributesList )
    { top.appendChild( XmlConnectionAttributes( attributes ).domElement( document.get() ) ); }

    document.replaceChild( top );
    {
        QFile qfile( file_ );
        if( !qfile.open( QIODevice::WriteOnly ) ) return;
        qfile.write( document.toByteArray() );
    }

    return;
}
