/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "ConnectionWidget.h"
#include "LineEditor.h"
#include "TreeView.h"
#include "GridLayout.h"
#include "IconEngine.h"
#include "IconNames.h"

#include <QGroupBox>
#include <QLabel>
#include <QLayout>

//___________________________________________________________
ConnectionWidget::ConnectionWidget( QWidget* parent ):
    QWidget(parent)
{

    Debug::Throw( "ConnectionWidget::ConnectionWidget.\n" );

    // vertical layout
    QVBoxLayout* vLayout = new QVBoxLayout( this );
    vLayout->setSpacing(5);
    vLayout->setMargin(0);
    setLayout( vLayout );

    // grid layout
    GridLayout* gridLayout( new GridLayout );
    gridLayout->setMargin(0);
    gridLayout->setSpacing(5);
    gridLayout->setMaxCount(4);
    gridLayout->setColumnAlignment(0, Qt::AlignRight|Qt::AlignVCenter );
    vLayout->addLayout( gridLayout );

    // name
    QLabel* label;
    gridLayout->addWidget( label = new QLabel( tr( "Name:" ), this ) );
    gridLayout->addWidget( nameEditor_ = new LineEditor( this ), 0, 1, 1, 3 );
    label->setBuddy( nameEditor_ );

    gridLayout->addWidget( label = new QLabel( tr( "Host:" ), this ) );
    gridLayout->addWidget( hostEditor_ = new LineEditor( this ) );
    label->setBuddy( hostEditor_ );

    gridLayout->addWidget( label = new QLabel( tr( "Port:" ), this ) );
    gridLayout->addWidget( portSpinBox_ = new QSpinBox( this ) );
    portSpinBox_->setMinimum(0);
    portSpinBox_->setMaximum(10000);
    label->setBuddy( portSpinBox_ );

    gridLayout->addWidget( label = new QLabel( tr( "User:" ), this ) );
    gridLayout->addWidget( userEditor_ = new LineEditor( this ), 2, 1, 1, 3 );
    label->setBuddy( userEditor_ );

    gridLayout->addWidget( rememberPassword_ = new QCheckBox( tr( "Remember password" ), this ), 3, 1, 1, 3 );
    gridLayout->addWidget( autoConnect_ = new QCheckBox( tr( "Connect at startup" ), this ), 4, 1, 1, 3 );
    autoConnect_->setToolTip( tr( "Check this option to start this connection automatically at startup" ) );

    // tunnelAttributes
    QGroupBox* groupBox = new QGroupBox( tr( "tunnelAttributes" ), this );
    vLayout->addWidget( groupBox );

    // horizontal layout
    QHBoxLayout* hLayout = new QHBoxLayout;
    hLayout->setSpacing(5);
    groupBox->setLayout( hLayout );

    hLayout->addWidget( list_ = new TreeView( groupBox ) );
    list_->setModel( &model_ );
    list_->setSortingEnabled( false );

    vLayout = new QVBoxLayout;
    vLayout->setMargin(0);
    vLayout->setSpacing(5);
    hLayout->addLayout( vLayout );

    // buttons
    vLayout->addWidget( addButton_ = new QPushButton( IconEngine::get( IconNames::Add ), tr( "Add" ), groupBox ) );
    vLayout->addWidget( removeButton_ = new QPushButton( IconEngine::get( IconNames::Remove ), tr( "Remove" ), groupBox ) );
    vLayout->addWidget( editButton_ = new QPushButton( IconEngine::get( IconNames::Edit ), tr( "Edit" ), groupBox ) );
    vLayout->addStretch( 1 );

    connect( removeButton_, SIGNAL(pressed()), SLOT(_remove()) );
    connect( list_, SIGNAL(activated(QModelIndex)), editButton_, SLOT(click()) );

    connect( list_->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)), SLOT(_updateButtons()) );
    connect( list_->selectionModel(), SIGNAL(currentChanged(QModelIndex,QModelIndex)), SLOT(_updateButtons()) );

    _updateButtons();

}


//________________________________________
void ConnectionWidget::setAttributes( const Ssh::ConnectionAttributes& attributes )
{
    Debug::Throw( "ConnectionWidget::setAttributes.\n" );

    // update editors
    nameEditor_->setText( attributes.name() );
    hostEditor_->setText( attributes.host() );
    portSpinBox_->setValue( attributes.port() );
    userEditor_->setText( attributes.userName() );
    rememberPassword_->setChecked( attributes.rememberPassword() );
    autoConnect_->setChecked( attributes.autoConnect() );

    // update model
    model_.set( attributes.tunnels().toList() );
    list_->resizeColumns();


}

//________________________________________
Ssh::ConnectionAttributes ConnectionWidget::attributes() const
{
    Debug::Throw( "ConnectionWidget::attributes.\n" );
    Ssh::ConnectionAttributes out;
    out.setName( nameEditor_->text() );
    out.setHost( hostEditor_->text() );
    out.setPort( portSpinBox_->value() );
    out.setUserName( userEditor_->text() );
    out.setRememberPassword( rememberPassword_->isChecked() );
    out.setAutoConnect( autoConnect_->isChecked() );
    out.setTunnels( model_.get() );

    return out;
}

//________________________________________
void ConnectionWidget::_updateButtons()
{
    Debug::Throw( "ConnectDialog::_updateButtons.\n" );

    bool hasSelection( !list_->selectionModel()->selectedRows().empty() );
    editButton_->setEnabled( hasSelection );
    removeButton_->setEnabled( hasSelection );

}

//________________________________________
void ConnectionWidget::_remove()
{

    Debug::Throw( "ConnectionWidget::_remove.\n" );

    // remove
    model_.remove( model_.get( list_->selectionModel()->selectedRows() ) );
    list_->resizeColumns();

}
