/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "MainMenu.h"

#include "Application.h"
#include "ConnectionParser.h"
#include "ConnectionObject.h"
#include "ConnectionListDialog.h"
#include "Debug.h"
#include "IconEngine.h"
#include "IconNames.h"
#include "IconSize.h"
#include "NetworkConnectionMonitor.h"
#include "QtUtil.h"
#include "Singleton.h"
#include "XmlOptions.h"

#include <QApplication>
#include <QStyleOption>
#include <QTimer>
#include <algorithm>

//____________________________________________________________
MainMenu::MainMenu( QWidget* parent ):
    QMenu( parent ),
    Counter( "MainMenu" )
{

    Debug::Throw( "MainMenu::MainMenu.\n" );

    _installActions();

    connect( qApp, SIGNAL(aboutToQuit()), SLOT(closeConnections()) );

    // title
    QtUtil::addMenuSection( this, windowIcon(), tr( "Connections" ) );

    addAction( configurationAction_ );
    addAction( reconnectAction_ );

    auto application( Base::Singleton::get().application<Application>() );
    addAction( &application->showLogAction() );

    addSeparator();

    // network connection monitor
    connectionMonitor_ = new Network::ConnectionMonitor( this );
    connectionMonitor_->checkDevice();
    connect( connectionMonitor_, SIGNAL(deviceConnected(QString)), SLOT(_reconnectAllDelayed()) );

    // configuration
    connect( Base::Singleton::get().application(), SIGNAL(configurationChanged()), SLOT(_updateConfiguration()) );
    _updateConfiguration();

}

//____________________________________________________________
void MainMenu::readConnections()
{

    Debug::Throw( "MainMenu::readConnections.\n" );
    ConnectionParser parser( File( XmlOptions::get().raw( "RC_FILE" ) ) );
    Ssh::ConnectionAttributes::List attributes = parser.read();
    if( parser.hasError() )
    { emit messageAvailable( parser.error() ); }

    if( attributes.empty() )
    {

        emit messageAvailable( tr( "No connections are presently registered.\n Righ-click to configure connections." ) );

    } else {

        for( const auto& connection:attributes )
        { _addConnection( connection ); }

    }

}

//____________________________________________________________
int MainMenu::connectionsCount() const
{ return findChildren<ConnectionObject*>().size(); }

//____________________________________________________________
void MainMenu::autoConnect()
{

    Debug::Throw( "MainMenu::autoConnect.\n" );

    // get attributes
    for( const auto& object:findChildren<ConnectionObject*>() )
    { if( object->attributes().autoConnect() ) object->connectAction().setChecked( true ); }

}

//____________________________________________________________
void MainMenu::connect( QString connectionName )
{

    Debug::Throw() << "MainMenu::connect - " << connectionName << endl;
    for( const auto& object:findChildren<ConnectionObject*>() )
    {
        if( object->attributes().name() == connectionName && !object->connectAction().isChecked() )
        {
            object->connectAction().setChecked( true );
            break;
        }
    }

}

//____________________________________________________________
void MainMenu::disconnect( QString connectionName )
{

    Debug::Throw() << "MainMenu::disconnect - " << connectionName << endl;
    for( const auto& object:findChildren<ConnectionObject*>() )
    {
        if( object->attributes().name() == connectionName && object->connectAction().isChecked() )
        {
            object->connectAction().setChecked( false );
            break;
        }
    }

}

//____________________________________________________________
void MainMenu::reconnect( QString connectionName )
{

    Debug::Throw() << "MainMenu::reconnect - " << connectionName << endl;
    for( const auto& object:findChildren<ConnectionObject*>() )
    {
        if( object->attributes().name() != connectionName ) continue;
        if( object->connectAction().isChecked() )
        {

            object->connectAction().setChecked( false );
            QMetaObject::invokeMethod( &object->connectAction(), "trigger", Qt::QueuedConnection );

        } else {

            object->connectAction().setChecked( true );

        }

        break;

    }

}

//____________________________________________________________
void MainMenu::reconnectExisting()
{

    Debug::Throw( "MainMenu::reconnectExisting.\n" );
    for( const auto& object:findChildren<ConnectionObject*>() )
    {
        if( object->connectAction().isChecked() )
        {
            object->connectAction().setChecked( false );
            QMetaObject::invokeMethod( &object->connectAction(), "trigger", Qt::QueuedConnection );
        }
    }

}

//____________________________________________________________
void MainMenu::reconnectAll()
{
    Debug::Throw( "MainMenu::reconnectAll.\n" );

    // reconnect existing connections
    for( const auto& object:findChildren<ConnectionObject*>() )
    {

        if( object->connectAction().isChecked() )
        {

            object->connectAction().setChecked( false );
            QMetaObject::invokeMethod( &object->connectAction(), "trigger", Qt::QueuedConnection );

        } else if( object->attributes().autoConnect() ) {

            object->connectAction().setChecked( true );

        }

    }
}

//____________________________________________________________
void MainMenu::closeConnections()
{
    Debug::Throw( "MainMenu::closeConnections.\n" );
    for( const auto& object:findChildren<ConnectionObject*>() )
    {  if( object->connectAction().isChecked() ) object->connectAction().setChecked( false ); }
}

//____________________________________________________________
void MainMenu::showEvent( QShowEvent* )
{

    Debug::Throw( "MainMenu::showEvent.\n" );
    bool isConnected( false );
    for( const auto& object:findChildren<ConnectionObject*>() )
    {
        if( object->connectAction().isChecked() )
        {
            isConnected = true;
            break;
        }
    }

    reconnectAction_->setEnabled( isConnected );

}

//__________________________________________________________
void MainMenu::_reconnectAllDelayed()
{ QTimer::singleShot( 500, this, SLOT(reconnectAll()) ); }

//__________________________________________________________
void MainMenu::_updateConfiguration()
{
    Debug::Throw( "MainMenu::_updateConfiguration.\n" );

    // device selection
    connectionMonitor_->setEnabled( XmlOptions::get().get<bool>( "NETWORK_AUTO_RECONNECT" ) );
    connectionMonitor_->setTimeOut( XmlOptions::get().get<int>( "NETWORK_AUTO_RECONNECT_TIMEOUT" ) );

}

//____________________________________________________________
void MainMenu::_saveConnections()
{

    Debug::Throw( "MainMenu::_saveConnections.\n" );
    Ssh::ConnectionAttributes::List attributes;

    // should respect ordering here
    QList<QAction*> actions( this->actions() );
    for( const auto& action:actions )
    {

        for( const auto& object:findChildren<ConnectionObject*>() )
        {
            if( &object->connectAction() == action )
            {
                attributes << object->attributes();
                break;
            }
        }

    }

    ConnectionParser parser( File( XmlOptions::get().raw( "RC_FILE" ) ) );
    parser.write( attributes );
    if( parser.hasError() )
    { emit messageAvailable( parser.error() ); }

    emit connectionStatusChanged();

}

//____________________________________________________________
void MainMenu::_configureConnections()
{
    Debug::Throw( "MainMenu::_configureConnections.\n" );

    // get attributes
    Ssh::ConnectionAttributes::List attributesList;

    auto objects( findChildren<ConnectionObject*>() );
    for( const auto& object:objects )
    { attributesList.append( object->attributes() ); }

    // create dialog
    ConnectionListDialog dialog( this );
    dialog.setAttributes( attributesList );
    if( !dialog.centerOnWidget( window() ).exec() ) return;

    attributesList = dialog.attributes();

    // delete all parent objects
    // this should be made smarter, in order not to delete objects for which identical attributes are found
    for( const auto& object:objects )
    {

        auto attributesIter(
            std::find_if(
            attributesList.begin(), attributesList.end(),
            SameConnectionFTor( object->attributes() ) ) );

        if( attributesIter == attributesList.end() )
        {

            // disconnect and delete connection
            object->connectAction().setChecked( false );
            object->deleteLater();

        } else {

            object->setAttributes( *attributesIter );
            attributesList.erase( attributesIter );
        }

    }

    for( const auto& attributes:attributesList )
    { _addConnection( attributes ); }

    // first remove all actions
    objects = findChildren<ConnectionObject*>();
    for( const auto& object:objects )
    { removeAction( &object->connectAction() ); }

    // change ordering
    // this can probably be made faster
    attributesList = dialog.attributes();
    for( const auto& attributes:attributesList )
    {
        for( const auto& object:objects )
        {
            if( object->attributes() == attributes )
            { addAction( &object->connectAction() ); }
        }
    }

    _saveConnections();
    autoConnect();

}

//____________________________________________________________
void MainMenu::_addConnection( const Ssh::ConnectionAttributes& attributes )
{

    Debug::Throw( "MainMenu::_addConnection.\n" );

    // create new object
    auto object = new ConnectionObject( this );
    object->setAttributes( attributes );
    connect( object, SIGNAL(messageAvailable(QString,Severity)), this, SIGNAL(messageAvailable(QString,Severity)) );

    connect( object, SIGNAL(connected()), this, SIGNAL(connectionStatusChanged()) );
    connect( object, SIGNAL(disconnected()), this, SIGNAL(connectionStatusChanged()) );
    connect( object, SIGNAL(attributesChanged()), SLOT(_saveConnections()) );

    addAction( &object->connectAction() );

}

//_______________________________________________________
void MainMenu::_installActions()
{

    // configuration
    configurationAction_ = new QAction( IconEngine::get( IconNames::Configure ), tr( "Configure Connections..." ), this );
    connect( configurationAction_, SIGNAL(triggered()), SLOT(_configureConnections()) );

    // reconnect action
    reconnectAction_ = new QAction( IconEngine::get( IconNames::Reload ), tr( "Reconnect" ), this ) ;
    reconnectAction_->setShortcut( QKeySequence::Refresh );
    connect( reconnectAction_, SIGNAL(triggered()), SLOT(reconnectExisting()) );

}

//_______________________________________________________
bool MainMenu::SameConnectionFTor::operator () (const Ssh::ConnectionAttributes& other ) const
{
    return
        attributes_.host() == other.host() &&
        attributes_.userName() == other.userName() &&
        attributes_.tunnels() == other.tunnels();
}
